/**
 * 
 */
package cotsbots.userballfollowing;

import android.util.Log;
import cotsbots.userballfollowing.SampleCvViewBase.Color;

/**
 * @author stinger
 * 
 */
public class Ball {
	public float mBallRangeWidthHeight = 2; // fraction that radius of
											// width has to
											// be within radius of.
											// Height 10 for +-1/10 2
											// for +-1/2
	public int GreenCorners = 1;
	public int leftEdge;
	public int rightEdge;
	public int topEdge;
	public int bottomEdge;
	public int m_radius;
	public int centerIdxWidth;
	public int centerIdxHeight;
	public int actualCenter;
	public boolean isSquare;
	public boolean isBall;
	public int rowMin;
	public int rowMax;
	public int colMin;
	public int colMax;
	public int centerRow;
	public int centerCol;

//	public Ball() {
	public Ball(int level) {
		switch (level) {
		case 0:
			//Random colored shape
			GreenCorners = 4;
			mBallRangeWidthHeight = 0.25f;
			break;
		case 1:
			GreenCorners = 3;
			mBallRangeWidthHeight = 2;
			break;
		case 2:
			GreenCorners = 2;
			mBallRangeWidthHeight = 2;
			break;
		case 3:
			//this is the setting used for research
			GreenCorners = 1;
			mBallRangeWidthHeight = 2;
			break;
		case 4:
			GreenCorners = 1;
			mBallRangeWidthHeight = 8;
			break;
		case 5:
			// Perfect Ball
			GreenCorners = 0;
			mBallRangeWidthHeight = 15;
			break;
		default:
			GreenCorners = 1;
			mBallRangeWidthHeight = 2;
			break;
		}
		ClearData();
	}

	public void ClearData() {
		leftEdge = 0;
		rightEdge = 0;
		topEdge = 0;
		bottomEdge = 0;
		m_radius = 0;
		centerIdxWidth = 0;
		centerIdxHeight = 0;
		actualCenter = 0;
		isSquare = false;
		rowMin = 0;
		rowMax = 0;
		colMin = 0;
		colMax = 0;
		centerRow = 0;
		centerCol = 0;
		isBall = false;
	}

	/**
	 * Calculates the center pixel of the ball
	 * 
	 * @param nFrameSize
	 *            = frame size of smaller RGB mat
	 * @param nCameraChannels
	 *            = camera channel count based on smaller RGB mat
	 */
	public void CalculateCenter(int nFrameSize, int nCameraChannels) {
		int col, row;
		col = centerIdxWidth % nFrameSize;
		row = centerIdxHeight / nFrameSize;

		actualCenter = row * nFrameSize + col;

		centerRow = row;// row index for 2d array

		centerCol = col / nCameraChannels; // find actual row index for 2d array
	}

	private static boolean isGood(int val) {
		if ((val % 4) == 0) {
			return true;
		}
		return false;
	}

	private double GetBallArea() {
		return Math.PI * m_radius * m_radius;
	}

	public double GetBallDistance() {
		double ratio = GetBallArea() / 102240.0;

		return Math.sqrt(0.0156 / ratio);
	}

	public double GetBallDistanceFast() {
		return 45.0 / (m_radius * 2);
	}

	/**
	 * Calculate the radius of the ball based on the RGB mat
	 * 
	 * @param nFramesize
	 *            = smaller RGB frame size
	 * @param nCameraChannels
	 *            = smaller RGB channel size (3)
	 */
	public void CalculateRadius(int nFrameSize, int nCameraChannels,
			int minRadius) {
		centerIdxWidth = (rightEdge + leftEdge) / 2;
		centerIdxWidth += isGood(centerIdxWidth) ? 0 : 2;
		int tmpB = bottomEdge / nFrameSize;
		int tmpT = topEdge / nFrameSize;
		int mod = topEdge % nFrameSize;
		int res = (tmpB + tmpT);
		res += (res % 2) == 0 ? 0 : 1;
		res /= 2;
		centerIdxHeight = res * nFrameSize + mod;

		int tmpWidth = (rightEdge - leftEdge) / (nCameraChannels * 2);
		int tmpHeight = (bottomEdge - topEdge) / (nFrameSize * 2);

		float range = (float)tmpWidth / (float)mBallRangeWidthHeight;

		m_radius = Math.max(tmpWidth, tmpHeight);
		if (m_radius > minRadius) {
			rowMin = (leftEdge % nFrameSize) / nCameraChannels;
			rowMax = (rightEdge % nFrameSize) / nCameraChannels;

			colMin = (topEdge / nFrameSize);
			colMax = (bottomEdge / nFrameSize);

			if (((tmpHeight + range) > (tmpWidth))
					&& ((tmpHeight - range) < tmpWidth)) {
				isSquare = true;
			} else {
				isSquare = false;
				Log.i("count", "not square");
			}
		} else {
			isSquare = false;
		}
	}

	public boolean checkBall(byte[] cameraBytes, Color col,
			int[] PixelInQuestion, int cameraChannels, int frameSize) {

		if (isSquare) {
			int topLeft = colMin * frameSize + rowMin * cameraChannels;
			int topRight = colMin * frameSize + rowMax * cameraChannels;
			int bottomLeft = colMax * frameSize + rowMin * cameraChannels;
			int bottomRight = colMax * frameSize + rowMax * cameraChannels;

			int tmpCounter = 0;
			PixelInQuestion[0] = ((int) cameraBytes[topLeft]) & 0xff;
			PixelInQuestion[1] = ((int) cameraBytes[topLeft + 1]) & 0xff;
			PixelInQuestion[2] = ((int) cameraBytes[topLeft + 2]) & 0xff;
			BallFollowingView.GetPixelHSV(PixelInQuestion);
			// BallFollowingView.GetPixelHSV(topLeft, PixelInQuestion,
			// cameraBytes);
			tmpCounter += BallFollowingView.isColor(col, PixelInQuestion) ? 1
					: 0;

			// BallFollowingView.GetPixelHSV(topRight, PixelInQuestion,
			// cameraBytes);
			PixelInQuestion[0] = ((int) cameraBytes[topRight]) & 0xff;
			PixelInQuestion[1] = ((int) cameraBytes[topRight + 1]) & 0xff;
			PixelInQuestion[2] = ((int) cameraBytes[topRight + 2]) & 0xff;
			BallFollowingView.GetPixelHSV(PixelInQuestion);
			tmpCounter += BallFollowingView.isColor(col, PixelInQuestion) ? 1
					: 0;

			// BallFollowingView.GetPixelHSV(bottomLeft, PixelInQuestion,
			// cameraBytes);
			PixelInQuestion[0] = ((int) cameraBytes[bottomLeft]) & 0xff;
			PixelInQuestion[1] = ((int) cameraBytes[bottomLeft + 1]) & 0xff;
			PixelInQuestion[2] = ((int) cameraBytes[bottomLeft + 2]) & 0xff;
			BallFollowingView.GetPixelHSV(PixelInQuestion);
			tmpCounter += BallFollowingView.isColor(col, PixelInQuestion) ? 1
					: 0;

			// BallFollowingView
			// .GetPixelHSV(bottomRight, PixelInQuestion, cameraBytes);
			PixelInQuestion[0] = ((int) cameraBytes[bottomRight]) & 0xff;
			PixelInQuestion[1] = ((int) cameraBytes[bottomRight + 1]) & 0xff;
			PixelInQuestion[2] = ((int) cameraBytes[bottomRight + 2]) & 0xff;
			BallFollowingView.GetPixelHSV(PixelInQuestion);
			tmpCounter += BallFollowingView.isColor(col, PixelInQuestion) ? 1
					: 0;

			// Are 3 out of 4 edges of colored object round? (all 4 edges should
			// be
			// round, but allowing 1 edge to be colored is a little more robust)

			isBall = tmpCounter > GreenCorners ? false : true;
			if (!isBall) {
				Log.i("count", "not round");
			}
			return isBall;
		} else {
			isBall = false;
			return false;
		}
	}

	public void Copy(Ball tmpBall) {
		this.leftEdge = tmpBall.leftEdge;
		this.rightEdge = tmpBall.rightEdge;
		this.bottomEdge = tmpBall.bottomEdge;
		this.topEdge = tmpBall.topEdge;
		this.m_radius = tmpBall.m_radius;
		this.centerIdxWidth = tmpBall.centerIdxWidth;
		this.centerIdxHeight = tmpBall.centerIdxHeight;
		this.actualCenter = tmpBall.actualCenter;
		this.isSquare = tmpBall.isSquare;
		this.isBall = tmpBall.isBall;
		this.rowMax = tmpBall.rowMax;
		this.rowMin = tmpBall.rowMin;
		this.colMin = tmpBall.colMin;
		this.colMax = tmpBall.colMax;
		this.centerRow = tmpBall.centerRow;
		this.centerCol = tmpBall.centerCol;
	}

	/**
	 * Draw a bounding box around a ball
	 * 
	 * @param cameraBytes
	 *            = actual array of bytes to draw bounding box on
	 * @param cameraChannels
	 *            = channel count for actual mat size
	 * @param frameSize
	 *            = frame size of actual mat size
	 */
	public void DrawBall(byte[] cameraBytes, int cameraChannels, int frameSize) {
		int idx = colMin * frameSize + rowMin * cameraChannels;
		int tmpIdx;
		tmpIdx = idx;
		for (int row = rowMin; row < rowMax; row++) {
			cameraBytes[tmpIdx] = -1;
			cameraBytes[tmpIdx + 1] = -1;
			cameraBytes[tmpIdx + 2] = -1;
			tmpIdx += cameraChannels;
		}
		idx += frameSize;

		int tmpWidth = rowMax - rowMin;
		for (int col = colMin; col < colMax; col++) {
			tmpIdx = idx;
			cameraBytes[tmpIdx] = -1;
			cameraBytes[tmpIdx + 1] = -1;
			cameraBytes[tmpIdx + 2] = -1;
			tmpIdx += (tmpWidth * cameraChannels);
			cameraBytes[tmpIdx] = -1;
			cameraBytes[tmpIdx + 1] = -1;
			cameraBytes[tmpIdx + 2] = -1;
			idx += frameSize;
		}

		cameraBytes[actualCenter] = -1;
		cameraBytes[actualCenter + 1] = -1;
		cameraBytes[actualCenter + 2] = -1;

		for (int row = rowMin; row < rowMax; row++) {
			cameraBytes[tmpIdx] = -1;
			cameraBytes[tmpIdx + 1] = -1;
			cameraBytes[tmpIdx + 2] = -1;
			tmpIdx -= cameraChannels;
		}

	}
}
