/**
 * BallFollowingAct.java
 * 
 * Ball Following activity used so robots can follow red, green, or blue balls with distance
 * calculated based on a ball size of roughly 3 inches in diameter.
 * 
 * 1. Download onto phone
 * 2. Search for Address first as all properties will be reset after this event
 * 3. Select remaining 3 properties based on preferences 
 * (object color, shape distortion, robot driving platform type)
 * Shape distortion specifies how closely to a ball shape the object to follow must be.
 * 	- Random selection makes any colored object a target
 * 	- Ball selection will only target ball shaped objects (This may have many false negatives
 *  	resulting in poor following behavior)
 *  - Default Shape Distortion level gives best ball following performance
 *  4. Place cell phone on robot
 *  5. Connect power to robot if not already on
 *  6. Click "Engine Start" button on phone
 *  
 *  NOTE: pressing menu button on phone during program execution will allow color selection
 *  
 * @author stinger
 */

package cotsbots.userballfollowing;

import cotsbots.devicedescovery.BluetoothDiscoveryActivity;
import cotsbots.graduate.robotcontroller.CotsBotsRobotController;
import cotsbots.userballfollowing.SampleCvViewBase.Color;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.RadioGroup;
import android.widget.SeekBar;
import android.widget.TextView;

public class BallFollowingAct extends Activity {
	static String s_MACAddress;
	static Color s_BallColor;

	// /////Member variables///////
	private ImageButton mStartRobotButton;
	private RadioGroup mColorRadioGroup;
	private RadioGroup mRadioGroupRobotType;
	private Activity mMainContext;
	private BallFollowingView mBallFollower;
	private MenuItem mRed;
	private MenuItem mGreen;
	private MenuItem mBlue;

	// Member variables used for starting bluetooth discovery activity
	private Button mSearchAddressButton;
	private Intent mIntent;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
				WindowManager.LayoutParams.FLAG_FULLSCREEN);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		setContentView(R.layout.main);
		mIntent = new Intent(this, BluetoothDiscoveryActivity.class);
		setGUIViews();
	}

	private void setGUIViews() {
		mMainContext = this;
		mSearchAddressButton = (Button) findViewById(R.id.buttonSearchAddress);
		mSearchAddressButton.setOnClickListener(SearchAddressListener);
		mStartRobotButton = (ImageButton) findViewById(R.id.imageButtonStart);
		mStartRobotButton.setOnClickListener(StartRobotListener);
		mColorRadioGroup = (RadioGroup) findViewById(R.id.radioGroup1);
		mRadioGroupRobotType = (RadioGroup) findViewById(R.id.radioGroupRobotType);
	}

	private OnClickListener SearchAddressListener = new OnClickListener() {

		@Override
		public void onClick(View v) {
			startActivityForResult(mIntent,
					BluetoothDiscoveryActivity.sf_REQUEST_CODE_BLUETOOTH);
		}
	};

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == BluetoothDiscoveryActivity.sf_REQUEST_CODE_BLUETOOTH) {
			setContentView(R.layout.main);
			setGUIViews();
			if (resultCode == RESULT_OK) {
				ChangeAddress(data
						.getStringExtra(BluetoothDiscoveryActivity.sf_SELECTED_MAC_ADDRESS));
			}
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		mRed = menu.add("Red Ball");
		mBlue = menu.add("Blue Ball");
		mGreen = menu.add("Green Ball");
		return true;
	}

	@Override
	protected void onPause() {
		super.onPause();
		if (mBallFollower != null) {
			mBallFollower.robotBrain.mController.disconnect();
		}
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		if (item == mRed) {
			s_BallColor = Color.RED;
			if (mBallFollower != null) {
				mBallFollower.mBallColor = s_BallColor;
			}
		} else if (item == mBlue) {
			s_BallColor = Color.BLUE;
			if (mBallFollower != null) {
				mBallFollower.mBallColor = s_BallColor;
			}
		} else if (item == mGreen) {
			s_BallColor = Color.GREEN;
			if (mBallFollower != null) {
				mBallFollower.mBallColor = s_BallColor;
			}
		}
		return true;
	}

	public OnClickListener StartRobotListener = new OnClickListener() {

		@Override
		public void onClick(View arg0) {
			if (s_MACAddress.length() != BluetoothDiscoveryActivity.sf_MAC_ADDRESS_LENGTH) {
				return;
			}

			if (R.id.radioRed == mColorRadioGroup.getCheckedRadioButtonId()) {
				s_BallColor = Color.RED;
			} else if (R.id.radioBlue == mColorRadioGroup
					.getCheckedRadioButtonId()) {
				s_BallColor = Color.BLUE;
			} else {
				s_BallColor = Color.GREEN;
			}

			SeekBar barDistortionLevel = (SeekBar) findViewById(R.id.seekBar1);
			int value = barDistortionLevel.getProgress();

			CotsBotsRobotController.SteeringType steeringType = CotsBotsRobotController.SteeringType.SERVO;
			if (R.id.robotTypeTracked == mRadioGroupRobotType
					.getCheckedRadioButtonId()) {
				steeringType = CotsBotsRobotController.SteeringType.TRACKED;
			}

			mBallFollower = new BallFollowingView(mMainContext, s_BallColor,
					value, s_MACAddress, steeringType);
			setContentView(mBallFollower);
		}
	};

	private void ChangeAddress(String newAddress) {
		s_MACAddress = newAddress;
		TextView macAddressView = (TextView) findViewById(R.id.macTextView);
		macAddressView.setText(newAddress);
	}
}