/**
 * 
 */
package cotsbots.userballfollowing;

import cotsbots.graduate.robotcontroller.CotsBotsRobotController;
import cotsbots.graduate.robotcontroller.CotsBotsRobotController.SteeringType;
import cotsbots.userballfollowing.ScreenData.ScreenLocation;

/**
 * @author stinger
 * 
 */
public class BallFollowingBrain {
	private static final String sFORWARD = "Drive Forward";
	private static final String sBACKWARD = "Drive Backward";
	private static final String sLEFT = "Turn Left";
	private static final String sRIGHT = "Turn Right";
	private static final String sSTOP = "Stop";
	private static final double sf_BallFarDistance = 2.0;
	private static final double sf_BallMediumDistance = 0.6;
	private static final double sf_BallCloseDistance = 0.4;

	private static final int MAX_TURN_TIME = 25;
	public static String sRobotInstruction = "";
	public static String sBallDirection = "";
	public static double sRobotThrottle;
	public static double sBallDistance = 0.0;

	ScreenData mPreviousBallData;
	private ScreenData mData;
	public CotsBotsRobotController mController;
	public String mBluetoothMACAddress;
	private ScreenLocation mPreviousBallLocation = ScreenLocation.NONE;
	private int mTimer = 0;
	double mTimerSeconds = 0;
	Stopwatch mWatch;
	final double fMaxDistanceTimerSeconds = 1;
	DirectionVector mDirectionVector;
	double mThrottle = 1.0;
	double mTurningThrottle = 1.0;
	double mThrottleStep = 0.1;
	CotsBotsRobotController.SteeringType mSteeringType;

	public BallFollowingBrain(String macAddress, CotsBotsRobotController.SteeringType servoStyleSteering) {
		mSteeringType = servoStyleSteering;
		mData = new ScreenData();
		mDirectionVector = new DirectionVector();
		mPreviousBallData = new ScreenData();
		this.mBluetoothMACAddress = macAddress;
		mController = new CotsBotsRobotController(mBluetoothMACAddress);
		mWatch = new Stopwatch();
	}

	/**
	 * Decides what to do based on new ball data and returns whether or not the
	 * view should draw the ball
	 * 
	 * @param ball
	 * @return true if ball should be drawn
	 */
	public boolean DecideAction(Ball ball) {
		// hold a pointer to current ball
		// Does the view think it found a ball?
		if (ball.isBall) {
			// BallFollowingView found a ball like object
			// Determine ball location relative to phone
			mData.calculatePixelInformation(ball);
			mPreviousBallLocation = mData.GetBallLocation();
			// what way is ball headed relative to me
			mDirectionVector.updateDirection(mData.turnValue);

			mPreviousBallData.Copy(mData);
			// set text driving instruction on screen
			CreateDrivingInstruction(mPreviousBallData.MaxBallLocation,
					mData.mBallDistance);

			mTimer = 0;
			// Ball was found, now determine what to do with robot
			if (mData.mBallDistance > sf_BallFarDistance) {
				ControlRobot(1);
				//ControlRobot(mData.leftValue, mData.rightValue);
			} else if (mData.mBallDistance > sf_BallMediumDistance) {
				ControlRobot(Math.min(mData.mBallDistance,1));
			} else if (mData.mBallDistance < sf_BallCloseDistance) {
				
				ControlRobot(mData.mBallDistance);
				//ControlRobot(-1 * mData.rightValue, -1 * mData.leftValue);
			} else {
				// Robot is at stopping distance of ball, don't move
				ControlRobot(0.5);
				//ControlRobot(0, 0);
			}
			// Setup text to be displayed on screen for user 
			sBallDirection = mDirectionVector.getDirection() + "";
			sRobotThrottle = mThrottle;
			sBallDistance = mPreviousBallData.mBallDistance;
			return true;
		} else {
			// Robot does not see ball on his screen
			
			mDirectionVector.clearSpeed();
			mThrottle = 1.0;
			sRobotThrottle = mThrottle;
			sBallDistance = 0;
			CreateDrivingInstruction(ScreenLocation.NONE, 0.4);
			sBallDirection = mDirectionVector.getDirection() + "";
				if (TryFindBall2()) {
					if (mSteeringType == CotsBotsRobotController.SteeringType.TRACKED){
						ControlRobot(mPreviousBallLocation);
					}
			} else {
				mPreviousBallLocation = ScreenLocation.NONE;
				ControlRobot(mPreviousBallLocation);
			}
			return false;
		}
	}

	private boolean TryFindBall2() {
		if (((mPreviousBallLocation == ScreenLocation.LEFT) || (mPreviousBallLocation == ScreenLocation.RIGHT))
				&& (mTimer == 0)) {
			mTimer = MAX_TURN_TIME;
		}
		if (mTimer > 0) {
			mTimer--;
			if (mTimer == 0) {
				mPreviousBallLocation = ScreenLocation.NONE;
			}
			return true;
		}
		return false;
	}

	private void CreateDrivingInstruction(ScreenLocation location,
			double distance) {
		switch (location) {
		case LEFT:
			sRobotInstruction = sLEFT;
			break;
		case CENTER:
			if (distance > sf_BallMediumDistance) {
				sRobotInstruction = sFORWARD;
			} else if (distance < sf_BallCloseDistance) {
				sRobotInstruction = sBACKWARD;
			} else {
				sRobotInstruction = sSTOP;
			}
			break;
		case RIGHT:
			sRobotInstruction = sRIGHT;
			break;
		case NONE:
			sRobotInstruction = sSTOP;
			break;
		}
	}
	
	public void ControlRobot(double speed) {
		if (speed < .5) {
			// If robot is extremely close to ball, then backup using the reverse
			// values for left and right (leftValue goes in rightValue's location
			// and vice versa)
			if (mSteeringType == SteeringType.SERVO){
				mData.turnValue = 1 - mData.turnValue;
			}
		}
		mController.driveVehicle(mData.turnValue, speed, mSteeringType);
	}

	public void ControlRobot(ScreenLocation location) {
		switch (location) {
		case LEFT:
			mController.driveVehicle(.3, .65, mSteeringType);
			break;
		case CENTER:
			mController.driveVehicle(.5, 1, mSteeringType);
			break;
		case RIGHT:
			mController.driveVehicle(.8, .65, mSteeringType);
			// controller.turnRight();
			break;
		case NONE:
			mController.driveVehicle(.5, .5, mSteeringType);
			// controller.stop();
			break;
		case BACKWARD:
			mController.driveVehicle(.5, 0, mSteeringType);
		}
	}
}
