package cotsbots.userballfollowing;

import java.util.ArrayList;

import org.opencv.android.Utils;
import org.opencv.core.Mat;
import org.opencv.highgui.Highgui;
import org.opencv.highgui.VideoCapture;

import cotsbots.graduate.robotcontroller.CotsBotsRobotController;

import android.app.Activity;
import android.content.pm.ActivityInfo;
import android.view.SurfaceHolder;

class BallFollowingView extends SampleCvViewBase {
	private static final int CrossSize = 14;
	private static final int MinRadius = 5;

	private static final int CameraChannels = 4; // RGBA is 4 bytes
	private final static int leftEdge = mCameraWidth / 3;
	private final static int rightEdge = leftEdge * 2;
	private final static int FrameSize = mCameraWidth * CameraChannels;

	private final static int FrameSizeColStep = FrameSize * CrossSize;
	private final static int FrameSizeRowStep = CameraChannels * CrossSize;
	private final static int StartIdx = FrameSize * ((int) (CrossSize / 2))
			+ ((int) (CrossSize / 2) * CameraChannels);

	private Mat mRgba = new Mat();

	public Color mBallColor;

	private int mArraySize;
	private byte[] CameraBytes;
	private int[] PixelInQuestion = new int[3];
	private Ball CurrentBall;
	private Ball tmpBall;

	public BallFollowingBrain robotBrain;
	private ArrayList<NonBallColorObjects> mNonBallObjs = new ArrayList<NonBallColorObjects>();

	public BallFollowingView(Activity context, Color ballColor, int ballDistortionLevel,
			String MACAddress, CotsBotsRobotController.SteeringType servoStyleSteering) {
		super(context);
		context.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		CurrentBall = new Ball(ballDistortionLevel);
		tmpBall = new Ball(ballDistortionLevel);
		this.mBallColor = ballColor;

		mArraySize = mCameraWidth * CameraHeight * CameraChannels;
		CameraBytes = new byte[mArraySize];

		robotBrain = new BallFollowingBrain(MACAddress, servoStyleSteering);
	}

	@Override
	public void surfaceChanged(SurfaceHolder _holder, int format, int width,
			int height) {
		super.surfaceChanged(_holder, format, width, height);
	}

	@Override
	protected void processFrame(VideoCapture capture) {
		capture.retrieve(mRgba, Highgui.CV_CAP_ANDROID_COLOR_FRAME_RGBA);
		findBall();
	}

	private void findBall() {
		// Get byte stream from matrix
		mRgba.get(0, 0, CameraBytes);

		int idxf = StartIdx;
		int idxff;
		int rows = mRgba.rows();
		int cols = mRgba.cols();

		for (int row = CrossSize / 2; row < rows - CrossSize; row += CrossSize) {
			idxff = idxf;
			for (int col = CrossSize / 2; col < cols - CrossSize; col += CrossSize) {
				if (!AlreadyExploredArea(idxff)) {
					PixelInQuestion[0] = ((int) CameraBytes[idxff]) & 0xff;
					PixelInQuestion[1] = ((int) CameraBytes[idxff + 1]) & 0xff;
					PixelInQuestion[2] = ((int) CameraBytes[idxff + 2]) & 0xff;
					GetPixelHSV(PixelInQuestion);
					if (isColor(mBallColor, PixelInQuestion)) {
						CheckSurrounding(idxff, mBallColor);
					}
				}
				idxff += FrameSizeRowStep;
			}
			idxf += FrameSizeColStep;
		}

		if (robotBrain.DecideAction(CurrentBall)) {
			// Draw ball bounding box
			CurrentBall.DrawBall(CameraBytes, CameraChannels, FrameSize);
		}
		

		DrawImageSections();
		// Put byte stream back into matrix
		mRgba.put(0, 0, CameraBytes);
		// Clear color of interest objects
		ResetData();
		// Convert matrix to bitmap
		Utils.matToBitmap(mRgba, mBmpCanvas);
	}

	private void ResetData() {
		CurrentBall.ClearData();
		mNonBallObjs.clear();
	}

	private boolean AlreadyExploredArea(int idx) {
		int row = (idx % FrameSize) / CameraChannels;
		int col = (idx / FrameSize);

		for (NonBallColorObjects obj : mNonBallObjs) {
			if ((obj.rowMin <= row) && (obj.rowMax >= row)
					&& (obj.colMin <= col) && (obj.colMax >= col)) {
				return true;
			}
		}

		return BallAlreadyCovers(idx);
	}

	public static boolean isColorRGB(Color col, PixelHSV PixelInQuestion) {
		int distance;
		switch (col) {
		case RED:
			if (PixelInQuestion.h > 90) {
				distance = 50 + 2 * Math.abs(PixelInQuestion.s
						- PixelInQuestion.v);
				if (((PixelInQuestion.h - distance) > PixelInQuestion.s)
						&& ((PixelInQuestion.h - distance) > PixelInQuestion.v)) {
					return true;
				}
			}
			break;
		case GREEN:
			if (PixelInQuestion.s > (60 + (PixelInQuestion.h + PixelInQuestion.v) / 4)) {
				return true;
			}
			// else if (){
			// return true;
			// }
		case BLUE:
			return false;
		}
		return false;
	}

	public static void GetPixelRGB(int counter, PixelHSV PixelInQuestion,
			byte[] CameraBytes) {
		PixelInQuestion.h = ((int) CameraBytes[counter]) & 0xff;
		PixelInQuestion.s = ((int) CameraBytes[counter + 1]) & 0xff;
		PixelInQuestion.v = ((int) CameraBytes[counter + 2]) & 0xff;
	}

	public static boolean isColor(Color col, int[] PixelInQuestion) {
		switch (col) {
		case RED:
			// red original 20 && 155 ...... 15 160
			if ((PixelInQuestion[0] < 15) || (PixelInQuestion[0] > 160)) {
				if (PixelInQuestion[0] > 5 && PixelInQuestion[0] < 15) {
					if (PixelInQuestion[2] < 240) {
						return false;
					}
				}

				if (PixelInQuestion[1] < 50) {
					return false;
				}

				if (PixelInQuestion[2] < 50) {
					return false;
				}

				if (PixelInQuestion[1] < 120) {
					if (PixelInQuestion[2] < 250) {
						return false;
					}
				}
			} else {
				return false;
			}

			break;
		case GREEN:// 80-40 95-30 60-40......90-40
			if ((PixelInQuestion[0] < 95) && (PixelInQuestion[0] > 30)) {
				if (PixelInQuestion[1] < 70) {
					return false;
				}

				if (PixelInQuestion[2] < 60) {
					return false;
				}

				// if (PixelInQuestion.s < 50) {
				// if (PixelInQuestion.v < 165) {
				// return false;
				// }
				// }
			} else {
				return false;
			}
			break;
		case BLUE:
			// Blue original 128,100 130-85 120-85
			if ((PixelInQuestion[0] < 120) && (PixelInQuestion[0] > 85)) {
				if (PixelInQuestion[1] < 50) {
					return false;
				}

				if (PixelInQuestion[2] < 50) {
					return false;
				}

				if (PixelInQuestion[1] < 120) {
					if (PixelInQuestion[2] < 250) {
						return false;
					}
				}
			} else {
				return false;
			}
			break;
		}
		return true;
	}

	public static void GetPixelHSV(int[] PixelInQuestion) {
		double var_R, var_G, var_B, var_Min, var_Max, del_Max;
		double H = 0, S = 0, V = 0, del_R, del_G, del_B;
		// These are initially RGB values that we convert to hsv
		var_R = (double) (PixelInQuestion[0] / 255.0);
		var_G = (double) (PixelInQuestion[1] / 255.0);
		var_B = (double) (PixelInQuestion[2] / 255.0);

		var_Min = Math.min(var_R, Math.min(var_G, var_B));
		var_Max = Math.max(var_R, Math.max(var_G, var_B)); // Max. value of RGB
		del_Max = var_Max - var_Min; // Delta RGB value

		V = var_Max;

		if (del_Max == 0) // This is a gray, no chroma...
		{
			H = 0; // HSV results from 0 to 1
			S = 0;
		} else // Chromatic data...
		{
			S = del_Max / var_Max;

			del_R = ((((var_Max - var_R) / 6.0) + (del_Max / 2.0)) / del_Max);
			del_G = ((((var_Max - var_G) / 6.0) + (del_Max / 2.0)) / del_Max);
			del_B = ((((var_Max - var_B) / 6.0) + (del_Max / 2.0)) / del_Max);

			if (var_R == var_Max)
				H = del_B - del_G;
			else if (var_G == var_Max)
				H = (1.0 / 3.0) + del_R - del_B;
			else if (var_B == var_Max)
				H = (2.0 / 3.0) + del_G - del_R;

			if (H < 0)
				H += 1;
			if (H > 1)
				H -= 1;
		}

		PixelInQuestion[0] = (int) (H * 180);
		PixelInQuestion[1] = (int) (S * 255);
		PixelInQuestion[2] = (int) (V * 255);

		// return PixelInQuestion;
	}

	public enum BallType {
		LION, GAZELLE
	}

	private void DrawImageSections() {
		int idx = leftEdge * CameraChannels;
		for (int i = 0; i < CameraHeight; i++) {
			CameraBytes[idx] = 0;
			CameraBytes[idx + 1] = 0;
			CameraBytes[idx + 2] = 0;
			idx += FrameSize;
		}

		idx = rightEdge * CameraChannels;
		for (int i = 0; i < CameraHeight; i++) {
			CameraBytes[idx] = 0;
			CameraBytes[idx + 1] = 0;
			CameraBytes[idx + 2] = 0;
			idx += FrameSize;
		}
	}

	private void CheckSurrounding(int idx, Color col) {
		tmpBall.ClearData();
		tmpBall.leftEdge = findLeft(idx, col);
		tmpBall.rightEdge = findRight(idx, col);
		tmpBall.topEdge = findTop(idx, col);
		tmpBall.bottomEdge = findBottom(idx, col);

		tmpBall.CalculateRadius(FrameSize, CameraChannels, MinRadius);
		tmpBall.CalculateCenter(FrameSize, CameraChannels);

		tmpBall.leftEdge = findLeft(tmpBall.actualCenter, col);
		tmpBall.rightEdge = findRight(tmpBall.actualCenter, col);
		tmpBall.topEdge = findTop(tmpBall.actualCenter, col);
		tmpBall.bottomEdge = findBottom(tmpBall.actualCenter, col);

		tmpBall.CalculateRadius(FrameSize, CameraChannels, MinRadius);
		tmpBall.CalculateCenter(FrameSize, CameraChannels);

		boolean ballCopied = false;
		if (tmpBall.m_radius > CurrentBall.m_radius) {
			if (tmpBall.checkBall(CameraBytes, mBallColor, PixelInQuestion,
					CameraChannels, FrameSize)) {
				ballCopied = true;
				if (!BallAlreadyCovers(tmpBall.actualCenter)) {
					CurrentBall.Copy(tmpBall);
				}
			}
		}

		if ((!ballCopied) && (tmpBall.m_radius > MinRadius)) {
			NonBallColorObjects tmpColoredObject = new NonBallColorObjects(
					tmpBall.rowMin, tmpBall.rowMax, tmpBall.colMin,
					tmpBall.colMax);
			mNonBallObjs.add(tmpColoredObject);
		}
	}

	private int findLeft(int idx, Color col) {
		// int row = idx / nFrameSize;
		// int minLocation = row * nFrameSize;
		int row = idx / FrameSize;
		int minLocation = row * FrameSize;
		boolean cont = true;
		int maxLeft = idx;
		int step = 2;
		int tmpIdx;
		while (cont) {
			// tmpIdx = idx - step * nCameraChannels;
			tmpIdx = idx - step * CameraChannels;
			if (tmpIdx >= minLocation) {
				PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
				PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
				PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
				GetPixelHSV(PixelInQuestion);
				// GetPixelHSV(tmpIdx);
				if (isColor(col, PixelInQuestion)) {
					maxLeft = tmpIdx;
					step *= 2;
				} else {
					step /= 2;
					// tmpIdx = idx - step * nCameraChannels + nCameraChannels;
					tmpIdx = idx - step * CameraChannels + CameraChannels;

					while (cont) {
						if (tmpIdx > minLocation) {
							// GetPixelHSV(tmpIdx);
							PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
							PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
							PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
							GetPixelHSV(PixelInQuestion);
							if (isColor(col, PixelInQuestion)) {
								maxLeft = tmpIdx;
								// tmpIdx -= nCameraChannels;
								tmpIdx -= CameraChannels;
							} else {
								cont = false;
							}
						} else {
							cont = false;
						}
					}
				}
			} else {
				cont = false;
			}
		}
		return maxLeft;
	}

	private int findRight(int idx, Color col) {
		// int row = idx / nFrameSize;
		// int maxLocation = (row + 1) * nFrameSize;
		int row = idx / FrameSize;
		int maxLocation = (row + 1) * FrameSize;
		boolean cont = true;
		int maxRight = idx;
		int step = 2;
		int tmpIdx;
		while (cont) {
			// tmpIdx = idx + step * nCameraChannels;
			tmpIdx = idx + step * CameraChannels;
			if (tmpIdx < maxLocation) {
				// GetPixelHSV(tmpIdx);
				PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
				PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
				PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
				GetPixelHSV(PixelInQuestion);
				if (isColor(col, PixelInQuestion)) {
					maxRight = tmpIdx;
					step *= 2;
				} else {
					step /= 2;
					// tmpIdx = idx + step * nCameraChannels + nCameraChannels;
					tmpIdx = idx + step * CameraChannels + CameraChannels;

					while (cont) {
						if (tmpIdx < maxLocation) {
							// GetPixelHSV(tmpIdx);
							PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
							PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
							PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
							GetPixelHSV(PixelInQuestion);
							if (isColor(col, PixelInQuestion)) {
								maxRight = tmpIdx;
								// tmpIdx += nCameraChannels;
								tmpIdx += CameraChannels;
							} else {
								cont = false;
							}
						} else {
							cont = false;
						}
					}
				}
			} else {
				cont = false;
			}
		}
		return maxRight;
	}

	private int findTop(int idx, Color col) {
		boolean cont = true;
		int maxTop = idx;
		int step = 2;
		int tmpIdx;
		while (cont) {
			// tmpIdx = idx - nFrameSize * step;
			tmpIdx = idx - FrameSize * step;
			if (tmpIdx > 0) {
				// GetPixelHSV(tmpIdx);
				PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
				PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
				PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
				GetPixelHSV(PixelInQuestion);
				if (isColor(col, PixelInQuestion)) {
					maxTop = tmpIdx;
					step *= 2;
				} else {
					step /= 2;
					// tmpIdx = idx - nFrameSize;
					tmpIdx = idx - FrameSize;

					while (cont) {
						if (tmpIdx > 0) {
							// GetPixelHSV(tmpIdx);
							PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
							PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
							PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
							GetPixelHSV(PixelInQuestion);
							if (isColor(col, PixelInQuestion)) {
								maxTop = tmpIdx;
								// tmpIdx -= nFrameSize;
								tmpIdx -= FrameSize;
							} else {
								cont = false;
							}
						} else {
							cont = false;
						}
					}
				}
			} else {
				cont = false;
			}
		}
		return maxTop;
	}

	private int findBottom(int idx, Color col) {
		boolean cont = true;
		int maxBottom = idx;
		int step = 2;
		int tmpIdx;
		while (cont) {
			// tmpIdx = idx + nFrameSize * step;
			// if (tmpIdx < nCameraBytes.length) {
			tmpIdx = idx + FrameSize * step;
			if (tmpIdx < CameraBytes.length) {
				// GetPixelHSV(tmpIdx);
				PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
				PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
				PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
				GetPixelHSV(PixelInQuestion);
				if (isColor(col, PixelInQuestion)) {
					maxBottom = tmpIdx;
					step *= 2;
				} else {
					step /= 2;
					// tmpIdx = idx + nFrameSize;
					tmpIdx = idx + FrameSize;

					while (cont) {
						// if (tmpIdx < nCameraBytes.length) {
						if (tmpIdx < CameraBytes.length) {
							// GetPixelHSV(tmpIdx);
							PixelInQuestion[0] = ((int) CameraBytes[tmpIdx]) & 0xff;
							PixelInQuestion[1] = ((int) CameraBytes[tmpIdx + 1]) & 0xff;
							PixelInQuestion[2] = ((int) CameraBytes[tmpIdx + 2]) & 0xff;
							GetPixelHSV(PixelInQuestion);
							if (isColor(col, PixelInQuestion)) {
								maxBottom = tmpIdx;
								// tmpIdx += nFrameSize;
								tmpIdx += FrameSize;
							} else {
								cont = false;
							}
						} else {
							cont = false;
						}
					}
				}
			} else {
				cont = false;
			}
		}
		return maxBottom;
	}

	private boolean BallAlreadyCovers(int idx) {

		// int row = (idx % nFrameSize)
		// / nCameraChannels;
		// int col = (idx / nFrameSize);
		int row = (idx % FrameSize) / CameraChannels;
		int col = (idx / FrameSize);

		if ((CurrentBall.rowMin <= row) && (CurrentBall.rowMax >= row)
				&& (CurrentBall.colMin <= col) && (CurrentBall.colMax >= col)) {
			return true;
		}
		return false;
	}

}
