package cotsbots.userballfollowing;

import org.opencv.highgui.VideoCapture;
import org.opencv.highgui.Highgui;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public abstract class SampleCvViewBase extends SurfaceView implements
		SurfaceHolder.Callback, Runnable {
	public enum Color {
		RED, GREEN, BLUE
	}

	public static int mCameraWidth = 640;// 320;//640;////320;//640;// 576
	public static int CameraHeight = 480;// 240;//480;//240;//480;// 432

	private SurfaceHolder mHolder;
	private VideoCapture mCamera;
	private FpsMeter mFps;
	protected Bitmap mBmpCanvas = Bitmap.createBitmap(mCameraWidth,
			CameraHeight, Bitmap.Config.ARGB_8888);

	public SampleCvViewBase(Context context) {
		super(context);
		mHolder = getHolder();
		mHolder.addCallback(this);
		mFps = new FpsMeter();
	}

	public void surfaceChanged(SurfaceHolder _holder, int format, int width,
			int height) {
		synchronized (this) {
			if (mCamera != null && mCamera.isOpened()) {
				mCamera.set(Highgui.CV_CAP_PROP_FRAME_WIDTH, mCameraWidth);
				mCamera.set(Highgui.CV_CAP_PROP_FRAME_HEIGHT, CameraHeight);
			}
		}
	}

	public void surfaceCreated(SurfaceHolder holder) {
		mCamera = new VideoCapture(Highgui.CV_CAP_ANDROID);
		if (mCamera.isOpened()) {
			(new Thread(this)).start();
		} else {
			mCamera.release();
			mCamera = null;
		}
	}

	public void surfaceDestroyed(SurfaceHolder holder) {
		if (mCamera != null) {
			synchronized (this) {
				mCamera.release();
				mCamera = null;
			}
		}
	}

	protected abstract void processFrame(VideoCapture capture);

	public void run() {
		mFps.init();
		while (true) {
			synchronized (this) {
				if (mCamera == null)
					break;

				if (!mCamera.grab()) {
					break;
				}
				processFrame(mCamera);
				mFps.measure();
			}
			if (mBmpCanvas != null) {
				Canvas canvas = mHolder.lockCanvas();
				// canvas.drawColor(android.graphics.Color.BLACK);
				if (canvas != null) {
					// RectF f = new RectF(0,0, 640, 480);
					// canvas.drawBitmap(mBmpCanvas, null, f,null);
					canvas.drawBitmap(mBmpCanvas,
							(canvas.getWidth() - mBmpCanvas.getWidth()) / 2,
							(canvas.getHeight() - mBmpCanvas.getHeight()) / 2,
							null);
					mFps.draw(canvas, 100, 0);
					mHolder.unlockCanvasAndPost(canvas);
				}
			}
		}
	}

}
